#include "mainwindow.h"

#include <QApplication>
#include <QDebug>
#include <QElapsedTimer>
#include <QFile>
#include <QTextStream>
#include <QSettings>
#include <QTranslator>
#include <SingleApplication>
#include <QStyleFactory>
#include "infolabel.h"

// global environment
#include "global.h"
GlobalEnvironment gEnv;
#include "deviceconfig.h"
#include <QTimer>

//#define QT_NO_DEBUG_OUTPUT // uncomment for no debug info

int main(int argc, char *argv[])
{
    #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
    QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
    QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
    #endif

    qRegisterMetaType<QList<QPair<bool, QString>> >();

    QElapsedTimer time;
    time.start();

    QApplication::setStyle(QStyleFactory::create("Fusion"));
    QApplication::setStyle(new InfoProxyStyle(qApp->style()));

    SingleApplication a(argc, argv);

    // global
    QSettings appSettings("EvilRacingSettings.conf", QSettings::IniFormat);
    DeviceConfig deviceConfig;
    QTranslator translator;
    gEnv.pAppSettings = &appSettings;
    gEnv.pDeviceConfig = &deviceConfig;
    gEnv.pTranslator = &translator;
    gEnv.pApp_start_time = &time;

    MainWindow w;

    QObject::connect(&a, &SingleApplication::instanceStarted, [&w](){
        w.raise();
        w.activateWindow();
    });

    qDebug() << "Application startup time =" << gEnv.pApp_start_time->elapsed() << "ms";
    w.show();

    return a.exec();
}
