#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QLocale>

#include "hiddevice.h"
#include "reportconverter.h"

class AxesConfig;
class Flasher;

QT_BEGIN_NAMESPACE
namespace Ui {
class MainWindow;
}
QT_END_NAMESPACE

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

signals:
    void getConfigDone(bool success);
    void sendConfigDone(bool success);

public slots:

private slots:
    void showConnectDeviceInfo();
    void hideConnectDeviceInfo();
    void flasherConnected();
    void deviceFlasherController(bool isStartFlash);
    void getParamsPacket(uint8_t *buffer);

    void configReceived(bool success);
    void configSent(bool success);

    void hidDeviceList(const QList<QPair<bool, QString>> &deviceNames);
    void hidDeviceListChanged(int index);

    void interfaceStyleChanged(bool isDark);

    void loadDefaultConfig();

    void on_pushButton_ReadConfig_clicked();
    void on_pushButton_WriteConfig_clicked();

    void on_pushButton_SaveToFile_clicked();
    void on_pushButton_LoadFromFile_clicked();
    void on_pushButton_LoadDefaultConfig_clicked();

    void on_toolButton_AdvSet_clicked();

protected:

private:
    Ui::MainWindow *ui;
    QThread *m_thread;
    HidDevice *m_hidDeviceWorker;

    QThread *m_threadGetSendConfig;

    AxesConfig *m_axesConfig = nullptr;
    Flasher *m_flasher = nullptr;
    bool m_flasherConnected;
    bool m_oldFirmware;
    bool m_deviceChanged = false;

    //bool m_trickForRgCompatib = false;

    QString m_buttonDefaultStyle;

    void readFromConfig();
    void writeToConfig();
    void oldConfigHandler();

    QIcon pixmapToIcon(QPixmap pixmap, const QColor &color);
    void updateColor();

    void loadAppConfig();
    void saveAppConfig();

    void loadDeviceConfigFromFile(QSettings *deviceSettings);
    void saveDeviceConfigToFile(QSettings *deviceSettings);

    bool m_themeIsDark;
};
#endif // MAINWINDOW_H
