#include <QThread>
#include <QTimer>
#include <QSettings>
#include <QFileDialog>
#include <QMessageBox>
#include <QSpinBox>

#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "mousewheelguard.h"
#include "widgets/tabs/axesconfig.h"

#include "common_types.h"
#include "global.h"
#include "deviceconfig.h"

#include <QDebug>

void MainWindow::readFromConfig()
{
    // read axes config
    m_axesConfig->readFromConfig();
}

void MainWindow::writeToConfig()
{
    // write axes config
    m_axesConfig->writeToConfig();
}


// load default config
void MainWindow::loadDefaultConfig()
{
    // load default config // called after all widgets have been created
    readFromConfig();
}


// slot after receiving the config
void MainWindow::configReceived(bool success)
{
    m_buttonDefaultStyle = ui->pushButton_ReadConfig->styleSheet();
    static QString button_default_text = ui->pushButton_ReadConfig->text();    //???

    if (success == true) //&& m_trickForRgCompatib == false)
    {
        readFromConfig();

        // set firmware version
        QString str = QString::number(gEnv.pDeviceConfig->config.firmware_version, 16);
        if (str.size() == 4){
            ui->label_DeviceStatus->setText(tr("Device firmware") + " v" + str[0] + "." + str[1] + "." + str[2] + "b" + str[3]);
        }

        ui->pushButton_ReadConfig->setText(tr("Received"));
        ui->pushButton_ReadConfig->setStyleSheet(m_buttonDefaultStyle + "color: white; background-color: rgb(0, 128, 0);");
        QTimer::singleShot(1500, this, [&]{
            ui->pushButton_ReadConfig->setStyleSheet(m_buttonDefaultStyle);
            ui->pushButton_ReadConfig->setText(button_default_text);
            if (ui->comboBox_HidDeviceList->currentIndex() >= 0){
                ui->pushButton_ReadConfig->setEnabled(true);
                ui->pushButton_WriteConfig->setEnabled(true);
            }
        });
    //} else if (success == true && m_trickForRgCompatib == true) {
        //on_pushButton_WriteConfig_clicked();
    } else {
        ui->pushButton_ReadConfig->setText(tr("Error"));
        ui->pushButton_ReadConfig->setStyleSheet(m_buttonDefaultStyle + "color: white; background-color: rgb(200, 0, 0);");
        QTimer::singleShot(1500, this, [&]{
            ui->pushButton_ReadConfig->setStyleSheet(m_buttonDefaultStyle);
            ui->pushButton_ReadConfig->setText(button_default_text);
            if (ui->comboBox_HidDeviceList->currentIndex() >= 0){
                ui->pushButton_ReadConfig->setEnabled(true);
                ui->pushButton_WriteConfig->setEnabled(true);
            }
        });
    }
}

// slot after sending the config
void MainWindow::configSent(bool success)
{
    m_buttonDefaultStyle = ui->pushButton_ReadConfig->styleSheet();
    static QString button_default_text = ui->pushButton_WriteConfig->text();    //???

    if (success == true)
    {
        ui->pushButton_WriteConfig->setText(tr("Sent"));
        ui->pushButton_WriteConfig->setStyleSheet(m_buttonDefaultStyle + "color: white; background-color: rgb(0, 128, 0);");

        QTimer::singleShot(1500, this, [&]{
            ui->pushButton_WriteConfig->setStyleSheet(m_buttonDefaultStyle);
            ui->pushButton_WriteConfig->setText(button_default_text);
            if (ui->comboBox_HidDeviceList->currentIndex() >= 0){
                ui->pushButton_ReadConfig->setEnabled(true);
                ui->pushButton_WriteConfig->setEnabled(true);
            }
        });
    } else {
        ui->pushButton_WriteConfig->setText(tr("Error"));
        ui->pushButton_WriteConfig->setStyleSheet(m_buttonDefaultStyle + "color: white; background-color: rgb(200, 0, 0);");

        QTimer::singleShot(1500, this, [&]{
            ui->pushButton_WriteConfig->setStyleSheet(m_buttonDefaultStyle);
            ui->pushButton_WriteConfig->setText(button_default_text);
            if (ui->comboBox_HidDeviceList->currentIndex() >= 0){
                ui->pushButton_ReadConfig->setEnabled(true);
                ui->pushButton_WriteConfig->setEnabled(true);
            }
        });
    }
    //m_trickForRgCompatib = false;
}




                               /////////////////////////////////// Load config from file ///////////////////////////////////
// load device config from file
void MainWindow::loadDeviceConfigFromFile(QSettings* deviceSettings)
{
    qDebug()<<"LoadDeviceConfigFromFile() started";
    // уменьшение текста в коде
    dev_config_t* devC = &gEnv.pDeviceConfig->config;
    bool tmp;

    // load Device USB config from file
    deviceSettings->beginGroup("DeviceUsbConfig");

    devC->firmware_version = QString(deviceSettings->value("FirmwareVersion", devC->firmware_version).toString()).toUShort(&tmp ,16);
    std::string tmp_string = deviceSettings->value("DeviceName", devC->device_name).toString().toStdString();
    for (uint i = 0; i < sizeof(devC->device_name); i++) {
        if (i < tmp_string.size()){
            devC->device_name[i] = tmp_string[i];
        } else {
            devC->device_name[i] = '\0';
        }
    }
    deviceSettings->endGroup();

    // load Axes config from file
    for (int i = 0; i < CUSTOM_MAX_AXIS_NUM; ++i)
    {
        deviceSettings->beginGroup("AxesConfig_" + QString::number(i));
        devC->axis_config[i].calib_min = int16_t(deviceSettings->value("CalibMin", devC->axis_config[i].calib_min).toInt());
        devC->axis_config[i].calib_center = int16_t(deviceSettings->value("CalibCenter", devC->axis_config[i].calib_center).toInt());
        devC->axis_config[i].calib_max = int16_t(deviceSettings->value("CalibMax", devC->axis_config[i].calib_max).toInt());
        devC->axis_config[i].is_centered = uint8_t(deviceSettings->value("IsCentered", devC->axis_config[i].is_centered).toInt());

        devC->axis_config[i].percent = uint8_t(deviceSettings->value("Percent", devC->axis_config[i].percent).toInt());

        devC->axis_config[i].out_enabled = uint8_t(deviceSettings->value("OutEnabled", devC->axis_config[i].out_enabled).toInt());
        deviceSettings->endGroup();
    }

    readFromConfig();
    qDebug()<<"LoadDeviceConfigFromFile() finished";
}


                                            /////////////////////////////////// SAVE config to file ///////////////////////////////////
void MainWindow::saveDeviceConfigToFile(QSettings* deviceSettings)
{
    dev_config_t* devC = &gEnv.pDeviceConfig->config;
    qDebug()<<"SaveDeviceConfigToFile() started";
    writeToConfig();

    // save Device USB config to file
    deviceSettings->beginGroup("DeviceUsbConfig");
    deviceSettings->setValue("FirmwareVersion", QString::number(devC->firmware_version, 16));
    deviceSettings->setValue("DeviceName", devC->device_name);
    deviceSettings->endGroup();

    // save Axes config to file
    for (int i = 0; i < CUSTOM_MAX_AXIS_NUM; ++i)
    {
        deviceSettings->beginGroup("AxesConfig_" + QString::number(i));
        deviceSettings->setValue("CalibMin", devC->axis_config[i].calib_min);
        deviceSettings->setValue("CalibCenter", devC->axis_config[i].calib_center);
        deviceSettings->setValue("CalibMax", devC->axis_config[i].calib_max);
        deviceSettings->setValue("IsCentered", devC->axis_config[i].is_centered);

        deviceSettings->setValue("Percent", devC->axis_config[i].percent);

        deviceSettings->setValue("OutEnabled", devC->axis_config[i].out_enabled);
        deviceSettings->endGroup();
    }

    qDebug()<<"SaveDeviceConfigToFile() finished";
}
