#include "sliderprogressbar.h"
#include <QPainter>
#include <QLabel>
#include <QMouseEvent>
#include <QWheelEvent>
#include <QStyle>
#include <QStyleOptionSlider>

#include <QDebug>

////////////////////////////////////////////// Horizontal orientation not finished!!! ///////////////////////////////////

SliderProgressBar::SliderProgressBar(QWidget *parent)
    : QProgressBar(parent)
    , m_currColor(m_handleColor)
    , m_label(new QLabel(this))
    , m_value(0)
    , m_drag(false)
{
    m_label->setFont(QFont("MS Shell Dlg 2", 8, 100));
    m_label->setAlignment(Qt::AlignHCenter | Qt::AlignTop);
    m_label->setStyleSheet("background:transparent; color: rgb(240, 241, 242);");
    m_label->setNum(m_value);
    //m_label->setText(QString::number(m_value) + "%");
    m_label->setVisible(true);

    m_label->setMouseTracking(true);
    setMouseTracking(true);
}


void SliderProgressBar::setSliderValue(int val)
{
    m_value = qMax(m_min, qMin(m_max, val));
    m_label->setNum(m_value);
    update();
}

int SliderProgressBar::sliderValue() const
{
    return m_value;
}


void SliderProgressBar::paintEvent(QPaintEvent *event)
{
    QProgressBar::paintEvent(event);
    QPainter p(this);

    if (orientation() == Qt::Horizontal) {
        int pos = QStyle::sliderPositionFromValue(m_min, m_max, m_value, width() - m_handleThick, true);
        m_handle = QRect(pos, 0, m_handleThick, height());
        m_label->move(pos, height() / 2); // need fix
    } else {
        int pos = QStyle::sliderPositionFromValue(m_min, m_max, m_value, height() - m_handleThick, true);
        m_handle = QRect(0, pos, width(), m_handleThick);

        m_label->move(0, pos -2);
    }

    p.fillRect(m_handle, m_currColor);
    if (QWidget::palette().color(QWidget::backgroundRole()).value() < 80) {
        p.setPen(QColor(140,140,140));
    } else {
        p.setPen(QColor(40,40,40));
    }
    p.drawRect(m_handle.adjusted(0, 0, -1, -1));
}


inline int SliderProgressBar::pick(const QPoint &pt) const
{
    return orientation() == Qt::Horizontal ? pt.x() : pt.y();
}

int SliderProgressBar::pixelPosToRangeValue(int pos) const
{
    int sliderMin, sliderMax;

    if (orientation() == Qt::Horizontal) {
        sliderMin = 0;
        sliderMax = width() - m_handleThick;
    } else {
        sliderMin = 0;
        sliderMax = height() - m_handleThick;
    }

    return QStyle::sliderValueFromPosition(m_min, m_max, pos - sliderMin,
                                               sliderMax - sliderMin, true);
}


void SliderProgressBar::mousePressEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton && m_handle.contains(event->pos())) {
        m_currColor = m_handleMoveColor;
        m_drag = true;
        update();
    }
}

void SliderProgressBar::mouseReleaseEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton) {
        if (m_handle.contains(event->pos())) {
            m_currColor = m_handleHoverColor;
        } else {
            m_currColor = m_handleColor;
        }
        m_drag = false;
        update();
    }
}

void SliderProgressBar::mouseMoveEvent(QMouseEvent *event)
{
    if (!m_drag && m_handle.contains(event->pos())) {
        m_currColor = m_handleHoverColor;
    } else if (m_drag) {
        m_value = pixelPosToRangeValue(pick(event->pos()));
        m_label->setNum(m_value);
    } else {
        m_currColor = m_handleColor;
    }

    update(); // optimize
}

void SliderProgressBar::wheelEvent(QWheelEvent *event)
{
    QPoint numPixels = event->pixelDelta();
    QPoint numDegrees = event->angleDelta() / 8;

    if (!numPixels.isNull()) {
        //scrollWithPixels(numPixels);  // for mac, not finished
    } else if (!numDegrees.isNull()) {
        QPoint numSteps = numDegrees / 15;
        scrollWithDegrees(numSteps);
    }

    event->accept();
}

void SliderProgressBar::scrollWithDegrees(QPoint numSteps)
{
    setSliderValue(sliderValue() + numSteps.y());
}

void SliderProgressBar::resizeEvent(QResizeEvent *event)
{
    Q_UNUSED(event)
    if (orientation() == Qt::Horizontal) {
        m_label->setFixedSize(20, 12);
    } else {
        m_label->setFixedSize(width(), 12);
    }
}

void SliderProgressBar::leaveEvent(QEvent *event)
{
    Q_UNUSED(event)
    if (!m_drag) {
        m_currColor = m_handleColor;
    }
}

