#include "axes.h"
#include "ui_axes.h"
#include <QTimer>
#include <QTranslator>
#include "converter.h"

#include <QDebug>
Axes::Axes(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::Axes)
{
    ui->setupUi(this);

    m_calibrationStarted = false;
    m_axisNumber = 0;
    ui->label_Raw->setNum(ui->progressBar_Raw->value());

    ui->progressBar_Out->setValue(0);
    int min = ui->progressBar_Out->minimum();
    int max = ui->progressBar_Out->maximum();
    int val = ui->progressBar_Out->value();
    ui->label_Output->setText(QString::number((val + 1 - min) * 100.0 / (max - min), 'f', 1) + "%");

    ui->progressBar_Raw->setValue(0);
    min = ui->progressBar_Raw->minimum();
    max = ui->progressBar_Raw->maximum();
    val = ui->progressBar_Raw->value();
    ui->label_Raw->setText(QString::number((val + 1 - min) * 100.0 / (max - min), 'f', 1) + "%");
}

Axes::~Axes()
{
    delete ui;
}

void Axes::retranslateUi()
{
    ui->retranslateUi(this);
}


void Axes::setAxisNumber(int number)
{
    if (number < 0) number = 0;
    else if (number > MAX_AXIS_NUM) number = MAX_AXIS_NUM;
    m_axisNumber = number;
}

void Axes::setAxisName(const QString &name)
{
    ui->groupBox_AxixName->setTitle(name);
}

void Axes::calibrationStarted(int rawValue)
{
    if (ui->spinBox_CalibMax->value() < rawValue) {
        ui->spinBox_CalibMax->setValue(rawValue);
    } else if (ui->spinBox_CalibMin->value() > rawValue) {
        ui->spinBox_CalibMin->setValue(rawValue);
    }
}

void Axes::on_pushButton_StartCalib_clicked(bool checked)
{
    m_calibrationStarted = checked;
    if (checked == true) {
        ui->pushButton_StartCalib->setText(m_kStopCalStr);
        ui->spinBox_CalibMax->setValue(AXIS_MIN_VALUE); // не перепутано
        ui->spinBox_CalibMin->setValue(AXIS_MAX_VALUE);

        connect(ui->progressBar_Raw, SIGNAL(valueChanged(int)), this, SLOT(calibrationStarted(int)));

    } else {
        disconnect(ui->progressBar_Raw, SIGNAL(valueChanged(int)), nullptr, nullptr);
        ui->pushButton_StartCalib->setText(m_kStartCalStr);

        ui->spinBox_CalibMax->setValue(ui->spinBox_CalibMax->value() * 99 / 100);   // deadzone after calibration 1%
        ui->spinBox_CalibMin->setValue(ui->spinBox_CalibMin->value() * 99 / 100);   // deadzone after calibration 1%
    }
}

void Axes::updateAxisRaw()
{
    int val = gEnv.pDeviceConfig->paramsReport.raw_axis_data[m_axisNumber];
    int min = ui->progressBar_Raw->minimum();
    int max = ui->progressBar_Raw->maximum();
    ui->label_Raw->setText(QString::number((val + 0 - min) * 100.0 / (max - min), 'f', 1) + "%");

    ui->progressBar_Raw->setValue(val);

}

void Axes::updateAxisOut()
{
    int val = gEnv.pDeviceConfig->paramsReport.axis_data[m_axisNumber];
    int min = ui->progressBar_Out->minimum();
    int max = ui->progressBar_Out->maximum();
    ui->label_Output->setText(QString::number((val + 0 - min) * 100.0 / (max - min), 'f', 1) + "%");

    ui->progressBar_Out->setValue(val);
}


void Axes::on_pushButton_ResetCalib_clicked()
{
    ui->spinBox_CalibMax->setValue(AXIS_MAX_VALUE);
    ui->spinBox_CalibMin->setValue(AXIS_MIN_VALUE);
}

void Axes::readFromConfig()
{
    axis_config_t *axCfg = &gEnv.pDeviceConfig->config.axis_config[m_axisNumber];
    // calibration
    ui->spinBox_CalibMin->setValue(axCfg->calib_min);
    ui->spinBox_CalibMax->setValue(axCfg->calib_max);
    // output percent
    ui->progressBar_Out->setSliderValue(axCfg->percent);
}

void Axes::writeToConfig()
{
    axis_config_t *axCfg = &gEnv.pDeviceConfig->config.axis_config[m_axisNumber];
    // calibration
    axCfg->calib_min = ui->spinBox_CalibMin->value();
    axCfg->calib_max = ui->spinBox_CalibMax->value();
    // output percent
    axCfg->percent = ui->progressBar_Out->sliderValue();
}
