#include "buttonconfig.h"
#include "ui_buttonconfig.h"
#include "buttonphysical.h"
#include "common_defines.h"
#include "common_types.h"
//#include <QTimer>
//#include <QSettings>
#include <QDebug>

static const int DEFAULT_BUTTONS_COUNT = 30;

ButtonConfig::ButtonConfig(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::ButtonConfig)
{
    ui->setupUi(this);
    ui->layoutG_Brightness->setAlignment(ui->verticalSlider_Brightness, Qt::AlignHCenter);
    m_shiftCheckBoxes << ui->checkBox_EncLeft << ui->checkBox_EncCenter << ui->checkBox_EncRight;

    setButtonsCount(DEFAULT_BUTTONS_COUNT);
}

ButtonConfig::~ButtonConfig()
{
    delete ui;
}

void ButtonConfig::retranslateUi()
{
    ui->retranslateUi(this);
}

void ButtonConfig::setButtonsCount(int count)
{
    // delete all
    while (!m_PhysButtonPtrList.empty()) {
        QWidget *widget = m_PhysButtonPtrList.takeLast();
        ui->layoutG_PhysicalButton->removeWidget(widget);
        widget->deleteLater();
    }
    // add
    int row = 0;
    int column = 0;
    int columnCount = 10;
    ui->layoutG_PhysicalButton->setAlignment(Qt::AlignCenter);
    for (int i = 0; i < count; i++) {
        if (column >= columnCount) // phys buttons column
        {
            row++;
            column = 0;
        }
        ButtonPhysical *physicalButtonWidget = new ButtonPhysical(i, this);
        ui->layoutG_PhysicalButton->addWidget(physicalButtonWidget, row, column);
        m_PhysButtonPtrList.append(physicalButtonWidget);
        column++;
    }

//    if (m_PhysButtonPtrList.isEmpty() == false) {
//        setMinimumWidth(columnCount * (m_PhysButtonPtrList[0]->size().width() + ui->layoutG_PhysicalButton->horizontalSpacing()));
//    }
}

void ButtonConfig::buttonStateChanged()
{
    int number = 0;
    params_report_t *paramsRep = &gEnv.pDeviceConfig->paramsReport;

    // logical buttons state
    for (int i = 0; i < 16; i++) {
        for (int j = 0; j < 8; j++) {
            number = j + (i *8);
            if (number < m_PhysButtonPtrList.size()) {
                if ((paramsRep->log_button_data[i] & (1 << (j & 0x07)))) {
                    m_PhysButtonPtrList[number]->setButtonState(true);
                } else if ((paramsRep->log_button_data[i] & (1 << (j & 0x07))) == false) {
                    m_PhysButtonPtrList[number]->setButtonState(false);
                }
            }
        }
    }
}


void ButtonConfig::updateRotaryButtons(QCheckBox *chBox, bool checked)
{
    int a = 0, b = 0;
    if (chBox == m_shiftCheckBoxes[0]) {
        a = 24;
        b = 25;
    } else if (chBox == m_shiftCheckBoxes[1]) {
        a = 26;
        b = 27;
    } else if (chBox == m_shiftCheckBoxes[2]) {
        a = 28;
        b = 29;
    }

    if (checked) {
        m_PhysButtonPtrList[a]->setDisabled(false);
        m_PhysButtonPtrList[b]->setDisabled(false);
    } else {
        m_PhysButtonPtrList[a]->setDisabled(true);
        m_PhysButtonPtrList[b]->setDisabled(true);
    }
}

void ButtonConfig::on_checkBox_EncLeft_toggled(bool checked)
{
    m_PhysButtonPtrList[10]->setDisabled(checked);
    updateRotaryButtons(ui->checkBox_EncLeft, checked);
}

void ButtonConfig::on_checkBox_EncCenter_toggled(bool checked)
{
    m_PhysButtonPtrList[11]->setDisabled(checked);
    updateRotaryButtons(ui->checkBox_EncCenter, checked);
}

void ButtonConfig::on_checkBox_EncRight_toggled(bool checked)
{
    m_PhysButtonPtrList[12]->setDisabled(checked);
    updateRotaryButtons(ui->checkBox_EncRight, checked);
}

void ButtonConfig::readFromConfig()
{
    int shiftsForRotary = 3;
    for (int i = 0; i < shiftsForRotary; ++i) {
        if (gEnv.pDeviceConfig->config.shift_config[i].button >= 0) {
            m_shiftCheckBoxes[i]->setChecked(true);
            updateRotaryButtons(m_shiftCheckBoxes[i], true);
        } else {
            m_shiftCheckBoxes[i]->setChecked(false);
            updateRotaryButtons(m_shiftCheckBoxes[i], false);
        }
    }

    ui->verticalSlider_Brightness->setValue(gEnv.pDeviceConfig->config.led_pwm_config[0].duty_cycle);
}

void ButtonConfig::writeToConfig()
{
    if (ui->checkBox_EncLeft->isChecked()) {
        gEnv.pDeviceConfig->config.shift_config[0].button = 10;
        gEnv.pDeviceConfig->config.buttons[10].is_disabled = true;
    } else {
        gEnv.pDeviceConfig->config.shift_config[0].button = -1;
        gEnv.pDeviceConfig->config.buttons[10].is_disabled = false;
    }

    if (ui->checkBox_EncCenter->isChecked()) {
        gEnv.pDeviceConfig->config.shift_config[1].button = 11;
        gEnv.pDeviceConfig->config.buttons[11].is_disabled = true;
    } else {
        gEnv.pDeviceConfig->config.shift_config[1].button = -1;
        gEnv.pDeviceConfig->config.buttons[11].is_disabled = false;
    }

    if (ui->checkBox_EncRight->isChecked()) {
        gEnv.pDeviceConfig->config.shift_config[2].button = 12;
        gEnv.pDeviceConfig->config.buttons[12].is_disabled = true;
    } else {
        gEnv.pDeviceConfig->config.shift_config[2].button = -1;
        gEnv.pDeviceConfig->config.buttons[12].is_disabled = false;
    }

    gEnv.pDeviceConfig->config.led_pwm_config[0].duty_cycle = ui->verticalSlider_Brightness->value();
}
