#include "translatebutton.h"
#include <QAction>
#include <QMenu>
#include "global.h"
#include <QTranslator>
#include <QDebug>

TranslateButton::TranslateButton(QWidget *parent) : QToolButton(parent)
{
    lang = QLocale::English;
    setIcon(QIcon(":/Images/flags/United_Kingdom.png"));
    createActions();
    createMenu();

    setMinimumHeight(33);
    setMenu(menu);
    setPopupMode(QToolButton::InstantPopup);
    setIconSize(QSize(22, 22));
}

void TranslateButton::setLanguage(QLocale::Language language)
{
    lang = language;
    switch (language) {
    case QLocale::English:
        translateEnglish();
        break;
    case QLocale::Russian:
        translateRussian();
        break;
    case QLocale::German:
        translateGerman();
        break;
    case QLocale::Czech:
        translateCzech();
        break;
    case QLocale::Swedish:
        translateSwedish();
        break;
    case QLocale::Finnish:
        translateFinnish();
        break;
    case QLocale::Slovak:
        translateSlovak();
        break;
    case QLocale::Hungarian:
        translateHungarian();
        break;
    default:
        ;
    }
}

QLocale::Language TranslateButton::currentLanguage() const
{
    return lang;
}

void TranslateButton::translateEnglish()
{
    setIcon(QIcon(":/Images/flags/United_Kingdom.png"));
    lang = QLocale::English;
    emit retranslated(QLocale::English);
}

void TranslateButton::translateRussian()
{
    setIcon(QIcon(":/Images/flags/Russia.png"));
    lang = QLocale::Russian;
    emit retranslated(QLocale::Russian);
}

void TranslateButton::translateGerman()
{
    setIcon(QIcon(":/Images/flags/Germany.png"));
    lang = QLocale::German;
    emit retranslated(QLocale::German);
}

void TranslateButton::translateCzech()
{
    setIcon(QIcon(":/Images/flags/Czech.png"));
    lang = QLocale::Czech;
    emit retranslated(QLocale::Czech);
}

void TranslateButton::translateSwedish()
{
    setIcon(QIcon(":/Images/flags/Sweden.png"));
    lang = QLocale::Swedish;
    emit retranslated(QLocale::Swedish);
}

void TranslateButton::translateFinnish()
{
    setIcon(QIcon(":/Images/flags/Finland.png"));
    lang = QLocale::Finnish;
    emit retranslated(QLocale::Finnish);
}

void TranslateButton::translateSlovak()
{
    setIcon(QIcon(":/Images/flags/Slovakia.png"));
    lang = QLocale::Slovak;
    emit retranslated(QLocale::Slovak);
}

void TranslateButton::translateHungarian()
{
    setIcon(QIcon(":/Images/flags/Hungary.png"));
    lang = QLocale::Hungarian;
    emit retranslated(QLocale::Hungarian);
}

void TranslateButton::createActions()
{
    actEnglish = new QAction(QIcon(":/Images/flags/United_Kingdom.png"), "English", this);
    connect(actEnglish, &QAction::triggered, this, &TranslateButton::translateEnglish);

    actRussian = new QAction(QIcon(":/Images/flags/Russia.png"), "Русский", this);
    connect(actRussian, &QAction::triggered, this, &TranslateButton::translateRussian);

    actGerman = new QAction(QIcon(":/Images/flags/Germany.png"), "Deutsche", this);
    connect(actGerman, &QAction::triggered, this, &TranslateButton::translateGerman);

    actCzech = new QAction(QIcon(":/Images/flags/Czech.png"), "Česky", this);
    connect(actCzech, &QAction::triggered, this, &TranslateButton::translateCzech);

    actSwedish = new QAction(QIcon(":/Images/flags/Sweden.png"), "Svenska", this);
    connect(actSwedish, &QAction::triggered, this, &TranslateButton::translateSwedish);

    actFinnish = new QAction(QIcon(":/Images/flags/Finland.png"), "Suomi", this);
    connect(actFinnish, &QAction::triggered, this, &TranslateButton::translateFinnish);

    actSlovak = new QAction(QIcon(":/Images/flags/Slovakia.png"), "Slovák", this);
    connect(actSlovak, &QAction::triggered, this, &TranslateButton::translateSlovak);

    actHungarian = new QAction(QIcon(":/Images/flags/Hungary.png"), "Magyar", this);
    connect(actHungarian, &QAction::triggered, this, &TranslateButton::translateHungarian);
}

void TranslateButton::createMenu()
{
    menu = new QMenu(this);
    menu->addAction(actEnglish);
    menu->addAction(actRussian);
    menu->addAction(actGerman);
    menu->addAction(actCzech);
    menu->addAction(actSwedish);
    menu->addAction(actFinnish);
    menu->addAction(actSlovak);
    menu->addAction(actHungarian);
}
